# SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
#
# SPDX-License-Identifier: Apache-2.0

from cuda.core._memoryview import (
    StridedMemoryView,  # noqa: F401
    args_viewable_as_strided_memory,  # noqa: F401
)
